@@ -29,7 +29,7 @@ conf_dir="../conf"
 lang_dir="../conf/vanilla/lang"
 fs_description="FreeSWITCH is a scalable open source cross-platform telephony platform designed to route and interconnect popular communication protocols using audio, video, text or any other form of media."
 mod_build_depends="." mod_depends="." mod_recommends="." mod_suggests="."
-supported_debian_distros="wheezy jessie stretch buster sid"
+supported_debian_distros="wheezy jessie stretch buster bullseye sid"
 supported_ubuntu_distros="trusty utopic xenial focal"
 supported_distros="$supported_debian_distros $supported_ubuntu_distros"
 avoid_mods=(
@@ -110,9 +110,10 @@ avoid_mods_focal=(
   applications/mod_cv
   event_handlers/mod_erlang_event
   event_handlers/mod_rayo
-  languages/mod_perl
   applications/mod_snom
   event_handlers/mod_snmp
+  asr_tts/mod_pocketsphinx
+  endpoints/mod_verto
 )
 manual_pkgs=(
 freeswitch-all
@@ -316,7 +317,7 @@ print_source_control () {
   esac
   local debhelper_dep="debhelper (>= 8.0.0)"
   if [ ${use_sysvinit} = "false" ]; then
-      debhelper_dep=${debhelper_dep}", dh-systemd"
+      debhelper_dep=${debhelper_dep}", dh-systemd | debhelper (>= 8.0.0)"
   fi
   cat <<EOF
 Source: freeswitch
@@ -342,15 +343,17 @@ Build-Depends:
 # configure options
  libssl1.0-dev | libssl-dev, unixodbc-dev, libpq-dev,
  libncurses5-dev, libjpeg62-turbo-dev | libjpeg-turbo8-dev | libjpeg62-dev | libjpeg8-dev,
- python-dev, python-all-dev, python-support (>= 0.90) | dh-python, erlang-dev, libtpl-dev (>= 1.5),
+ python-dev | python-dev-is-python2, python3-dev, python-all-dev, python-support (>= 0.90) | dh-python, erlang-dev, libtpl-dev (>= 1.5),
 # documentation
  doxygen,
 # for APR (not essential for build)
- uuid-dev, libexpat1-dev, libgdbm-dev, libdb-dev,
+ uuid-dev, libexpat1-dev, libgdbm-dev, libgdbm-compat-dev, libdb-dev,
 # used by many modules
  libcurl4-openssl-dev | libcurl4-gnutls-dev | libcurl-dev,
- bison, zlib1g-dev, libsofia-sip-ua-dev (>= 1.12.12),
+ bison, zlib1g-dev, libsofia-sip-ua-dev (>= 1.13.6),
  libspandsp3-dev,
+# used to format the private freeswitch apt-repo key properly
+ gnupg,
 # module build-depends
  $(debian_wrap "${mod_build_depends}")
 Standards-Version: 3.9.3
@@ -386,7 +389,7 @@ Description: Cross-Platform Scalable Multi-Protocol Soft Switch
 
 Package: libfreeswitch1
 Architecture: amd64 armhf
-Depends: \${shlibs:Depends}, \${misc:Depends}
+Depends: \${shlibs:Depends}, \${misc:Depends}, libsofia-sip-ua0 (>= 1.13.6)
 Recommends:
 Suggests: libfreeswitch1-dbg
 Conflicts: freeswitch-all (<= 1.6.7)
@@ -698,7 +701,7 @@ Depends: \${misc:Depends}, freeswitch (= \${binary:Version}),
  freeswitch-mod-java (= \${binary:Version}),
  freeswitch-mod-lua (= \${binary:Version}),
  freeswitch-mod-perl (= \${binary:Version}),
- freeswitch-mod-python (= \${binary:Version}),
+ freeswitch-mod-python3 (= \${binary:Version}),
  freeswitch-mod-yaml (= \${binary:Version}),
  freeswitch-mod-console (= \${binary:Version}),
  freeswitch-mod-logfile (= \${binary:Version}),
@@ -937,7 +940,7 @@ Depends: \${misc:Depends}, freeswitch (= \${binary:Version}),
  freeswitch-mod-java-dbg (= \${binary:Version}),
  freeswitch-mod-lua-dbg (= \${binary:Version}),
  freeswitch-mod-perl-dbg (= \${binary:Version}),
- freeswitch-mod-python-dbg (= \${binary:Version}),
+ freeswitch-mod-python3-dbg (= \${binary:Version}),
  freeswitch-mod-yaml-dbg (= \${binary:Version}),
  freeswitch-mod-console-dbg (= \${binary:Version}),
  freeswitch-mod-logfile-dbg (= \${binary:Version}),
