@@ -1,6 +1,6 @@
 /*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2015, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2021, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -353,7 +353,8 @@ typedef enum {
 	ED_BRIDGE_READ = (1 << 4),
 	ED_BRIDGE_WRITE = (1 << 5),
 	ED_TAP_READ = (1 << 6),
-	ED_TAP_WRITE = (1 << 7)
+	ED_TAP_WRITE = (1 << 7),
+	ED_STEREO = (1 << 8)
 } switch_eavesdrop_flag_enum_t;
 typedef uint32_t switch_eavesdrop_flag_t;
 
@@ -842,6 +843,9 @@ typedef enum {
 	SWITCH_RTP_FLAG_PASSTHRU,
 	SWITCH_RTP_FLAG_SECURE_SEND_MKI,
 	SWITCH_RTP_FLAG_SECURE_RECV_MKI,
+	SWITCH_RTP_FLAG_SRTP_HANGUP_ON_ERROR,
+	SWITCH_RTP_FLAG_AUDIO_FIRE_SEND_RTCP_EVENT,
+	SWITCH_RTP_FLAG_VIDEO_FIRE_SEND_RTCP_EVENT,
 	SWITCH_RTP_FLAG_INVALID
 } switch_rtp_flag_t;
 
@@ -1134,6 +1138,7 @@ typedef enum {
 	SWITCH_MESSAGE_INDICATE_DEFLECT,
 	SWITCH_MESSAGE_INDICATE_VIDEO_REFRESH_REQ,
 	SWITCH_MESSAGE_INDICATE_DISPLAY,
+	SWITCH_MESSAGE_INDICATE_MEDIA_PARAMS,
 	SWITCH_MESSAGE_INDICATE_TRANSCODING_NECESSARY,
 	SWITCH_MESSAGE_INDICATE_AUDIO_SYNC,
 	SWITCH_MESSAGE_INDICATE_VIDEO_SYNC,
@@ -1170,6 +1175,7 @@ typedef enum {
 	SWITCH_MESSAGE_RESAMPLE_EVENT,
 	SWITCH_MESSAGE_HEARTBEAT_EVENT,
 	SWITCH_MESSAGE_INDICATE_SESSION_ID,
+	SWITCH_MESSAGE_INDICATE_PROMPT,
 	SWITCH_MESSAGE_INVALID
 } switch_core_session_message_types_t;
 
@@ -1323,13 +1329,17 @@ typedef enum {
 } switch_core_session_message_flag_enum_t;
 typedef uint32_t switch_core_session_message_flag_t;
 
+typedef struct switch_core_session switch_core_session_t;
+static inline switch_core_session_t *switch_core_session_type_check(switch_core_session_t *session) { return session; }
+static inline const char *switch_const_char_type_check(const char *str) { return str; }
+
 #define SWITCH_CHANNEL_LOG SWITCH_CHANNEL_ID_LOG, __FILE__, __SWITCH_FUNC__, __LINE__, NULL
 #define SWITCH_CHANNEL_LOG_CLEAN SWITCH_CHANNEL_ID_LOG_CLEAN, __FILE__, __SWITCH_FUNC__, __LINE__, NULL
 #define SWITCH_CHANNEL_SESSION_LOG_CLEAN(x) SWITCH_CHANNEL_ID_LOG_CLEAN, __FILE__, __SWITCH_FUNC__, __LINE__, switch_core_session_get_uuid((x))
 #define SWITCH_CHANNEL_EVENT SWITCH_CHANNEL_ID_EVENT, __FILE__, __SWITCH_FUNC__, __LINE__, NULL
-#define SWITCH_CHANNEL_SESSION_LOG(x) SWITCH_CHANNEL_ID_SESSION, __FILE__, __SWITCH_FUNC__, __LINE__, (const char*)(x)
+#define SWITCH_CHANNEL_SESSION_LOG(x) SWITCH_CHANNEL_ID_SESSION, __FILE__, __SWITCH_FUNC__, __LINE__, (const char*)switch_core_session_type_check(x)
 #define SWITCH_CHANNEL_CHANNEL_LOG(x) SWITCH_CHANNEL_ID_SESSION, __FILE__, __SWITCH_FUNC__, __LINE__, (const char*)switch_channel_get_session(x)
-#define SWITCH_CHANNEL_UUID_LOG(x) SWITCH_CHANNEL_ID_LOG, __FILE__, __SWITCH_FUNC__, __LINE__, (x)
+#define SWITCH_CHANNEL_UUID_LOG(x) SWITCH_CHANNEL_ID_LOG, __FILE__, __SWITCH_FUNC__, __LINE__, switch_const_char_type_check(x)
 
 typedef enum {
 	CCS_DOWN,
@@ -1415,6 +1425,7 @@ CF_HOLD_BLEG        - B leg is on hold
 CF_SERVICE			- Channel has a service thread
 CF_TAGGED			- Channel is tagged
 CF_WINNER			- Channel is the winner
+CF_REUSE_CALLER_PROFILE - Channel reuse caller profile 
 CF_CONTROLLED		- Channel is under control
 CF_PROXY_MODE		- Channel has no media
 CF_SUSPEND			- Suspend i/o
@@ -1448,6 +1459,7 @@ typedef enum {
 	CC_IO_OVERRIDE,
 	CC_RTP_RTT,
 	CC_MSRP,
+	CC_MUTE_VIA_MEDIA_STREAM,
 	/* WARNING: DO NOT ADD ANY FLAGS BELOW THIS LINE */
 	CC_FLAG_MAX
 } switch_channel_cap_t;
@@ -1467,6 +1479,7 @@ typedef enum {
 	CF_SERVICE,
 	CF_TAGGED,
 	CF_WINNER,
+	CF_REUSE_CALLER_PROFILE,
 	CF_CONTROLLED,
 	CF_PROXY_MODE,
 	CF_PROXY_OFF,
@@ -1526,6 +1539,7 @@ typedef enum {
 	CF_INTERCEPTED,
 	CF_VIDEO_REFRESH_REQ,
 	CF_MANUAL_VID_REFRESH,
+	CF_MANUAL_MEDIA_PARAMS,
 	CF_SERVICE_AUDIO,
 	CF_SERVICE_VIDEO,
 	CF_ZRTP_PASSTHRU_REQ,
@@ -1559,6 +1573,7 @@ typedef enum {
 	CF_T38_PASSTHRU,
 	CF_DROP_DTMF,
 	CF_REINVITE,
+	CF_NOSDP_REINVITE,
 	CF_AUTOFLUSH_DURING_BRIDGE,
 	CF_RTP_NOTIMER_DURING_BRIDGE,
 	CF_AVPF,
@@ -1586,7 +1601,7 @@ typedef enum {
 	CF_VIDEO_DECODED_READ,
 	CF_VIDEO_DEBUG_READ,
 	CF_VIDEO_DEBUG_WRITE,
-	CF_VIDEO_ONLY,
+	CF_NO_RECOVER,
 	CF_VIDEO_READY,
 	CF_VIDEO_MIRROR_INPUT,
 	CF_VIDEO_READ_FILE_ATTACHED,
@@ -1620,6 +1635,10 @@ typedef enum {
 	CF_STATE_REPEAT,
 	CF_WANT_DTLSv1_2,
 	CF_RFC7329_COMPAT,
+	CF_REATTACHED,
+	CF_VIDEO_READ_TAPPED,
+	CF_VIDEO_WRITE_TAPPED,
+	CF_DEVICES_CHANGED,
 	/* WARNING: DO NOT ADD ANY FLAGS BELOW THIS LINE */
 	/* IF YOU ADD NEW ONES CHECK IF THEY SHOULD PERSIST OR ZERO THEM IN switch_core_session.c switch_core_session_request_xml() */
 	CF_FLAG_MAX
@@ -1897,7 +1916,8 @@ typedef enum {
 	SMBF_SPY_VIDEO_STREAM_BLEG = (1 << 23),
 	SMBF_READ_VIDEO_PATCH = (1 << 24),
 	SMBF_READ_TEXT_STREAM = (1 << 25),
-	SMBF_FIRST = (1 << 26)
+	SMBF_FIRST = (1 << 26),
+	SMBF_PAUSE = (1 << 27)
 } switch_media_bug_flag_enum_t;
 typedef uint32_t switch_media_bug_flag_t;
 
@@ -2034,6 +2054,7 @@ typedef uint32_t switch_io_flag_t;
     SWITCH_EVENT_SEND_INFO
     SWITCH_EVENT_RECV_INFO
     SWITCH_EVENT_RECV_RTCP_MESSAGE
+    SWITCH_EVENT_SEND_RTCP_MESSAGE
     SWITCH_EVENT_CALL_SECURE
     SWITCH_EVENT_NAT            	- NAT Management (new/del/status)
     SWITCH_EVENT_RECORD_START
@@ -2129,6 +2150,7 @@ typedef enum {
 	SWITCH_EVENT_SEND_INFO,
 	SWITCH_EVENT_RECV_INFO,
 	SWITCH_EVENT_RECV_RTCP_MESSAGE,
+	SWITCH_EVENT_SEND_RTCP_MESSAGE,
 	SWITCH_EVENT_CALL_SECURE,
 	SWITCH_EVENT_NAT,
 	SWITCH_EVENT_RECORD_START,
@@ -2232,7 +2254,12 @@ typedef enum {
 	SWITCH_CAUSE_DECLINE = 616,
 	SWITCH_CAUSE_DOES_NOT_EXIST_ANYWHERE = 617,
 	SWITCH_CAUSE_NOT_ACCEPTABLE = 618,
-	SWITCH_CAUSE_UNWANTED = 619
+	SWITCH_CAUSE_UNWANTED = 619,
+	SWITCH_CAUSE_NO_IDENTITY = 620,
+	SWITCH_CAUSE_BAD_IDENTITY_INFO = 621,
+	SWITCH_CAUSE_UNSUPPORTED_CERTIFICATE = 622,
+	SWITCH_CAUSE_INVALID_IDENTITY = 623,
+	SWITCH_CAUSE_STALE_DATE = 624
 } switch_call_cause_t;
 
 typedef enum {
@@ -2278,7 +2305,9 @@ typedef enum {
 	SCSC_SPS_PEAK,
 	SCSC_SPS_PEAK_FIVEMIN,
 	SCSC_SESSIONS_PEAK,
-	SCSC_SESSIONS_PEAK_FIVEMIN
+	SCSC_SESSIONS_PEAK_FIVEMIN,
+	SCSC_MDNS_RESOLVE,
+	SCSC_SHUTDOWN_CAUSE
 } switch_session_ctl_t;
 
 typedef enum {
@@ -2311,7 +2340,6 @@ typedef struct switch_rtcp_frame switch_rtcp_frame_t;
 typedef struct switch_channel switch_channel_t;
 typedef struct switch_sql_queue_manager switch_sql_queue_manager_t;
 typedef struct switch_file_handle switch_file_handle_t;
-typedef struct switch_core_session switch_core_session_t;
 typedef struct switch_caller_profile switch_caller_profile_t;
 typedef struct switch_caller_extension switch_caller_extension_t;
 typedef struct switch_caller_application switch_caller_application_t;
@@ -2794,6 +2822,7 @@ typedef struct switch_mm_s {
 	int cbr;
 	float fps;
 	float source_fps;
+	int source_kps;
 	int vbuf;
 	switch_video_profile_t vprofile;
 	switch_video_encode_speed_t vencspd;
@@ -2865,6 +2894,8 @@ typedef struct switch_dial_leg_s switch_dial_leg_t;
 struct switch_dial_leg_list_s;
 typedef struct switch_dial_leg_list_s switch_dial_leg_list_t;
 
+struct switch_dial_handle_list_s;
+typedef struct switch_dial_handle_list_s switch_dial_handle_list_t;
 
 SWITCH_END_EXTERN_C
 #endif
